@@ -7,12 +7,13 @@ import os.path as op
 import functools
 import warnings 
 import multiprocessing as mp 
+import copy 
 
 import numpy as np 
 import nibabel 
-import igl 
+import igl
+from numpy.lib.arraysetops import isin 
 from scipy import sparse
-
 try: 
     import pyvista
     import meshio 
@@ -22,8 +23,10 @@ except ImportError as e:
         " read/write VTK surfaces. VTK requires Python <=3.7 (as of May 2020)")
     _VTK_ENABLED = False 
 
+
 from .image_space import ImageSpace
 from .. import utils, core
+from ..utils import NP_FLOAT
 
 @utils.cascade_attributes
 def ensure_derived_space(func):
@@ -133,13 +136,14 @@ class Surface(object):
             # Finally, convert from voxel coords to world mm 
             ps = utils.affineTransformPoints(ps, struct_spc.vox2world)
 
-        self.points = ps.astype(np.float32)
+        self.points = ps.astype(NP_FLOAT)
         self.tris = ts.astype(np.int32)
         self.xProds = None 
         self.voxelised = None 
         self.name = name
         self.assocs = None 
         self._index_space = None 
+        self._use_mp = (self.tris.shape[0] > 1000)
 
     def __repr__(self):
 
@@ -163,12 +167,13 @@ class Surface(object):
             raise RuntimeError("ts array should be 0-indexed")
 
         s = cls.__new__(cls)
-        s.points = ps.astype(np.float32)
-        s.tris = ts.astype(np.int32)
+        s.points = copy.deepcopy(ps.astype(NP_FLOAT))
+        s.tris = copy.deepcopy(ts.astype(np.int32))
         s.xProds = None 
         s.voxelised = None 
         s.name = name
         s._index_space = None 
+        s._use_mp = (s.tris.shape[0] > 1000)
         return s
     
 
@@ -186,7 +191,7 @@ class Surface(object):
 
         gii = nibabel.GiftiImage()
         gii.add_gifti_data_array(
-            nibabel.gifti.GiftiDataArray(data.astype(np.float32)))
+            nibabel.gifti.GiftiDataArray(data.astype(NP_FLOAT)))
         nibabel.save(gii, path)
 
 
@@ -267,9 +272,9 @@ class Surface(object):
         surface's current index_space. 
         """
 
-        pvs_curr = self.voxelised.astype(np.float32)
+        pvs_curr = self.voxelised.astype(NP_FLOAT)
         pvs_curr[self.assocs_keys] = self.fractions
-        out = np.zeros(np.prod(space.size), dtype=np.float32)
+        out = np.zeros(np.prod(space.size), dtype=NP_FLOAT)
         curr_inds, dest_inds = self.reindexing_filter(space)
         out[dest_inds] = pvs_curr[curr_inds]
         return out.reshape(space.size)
@@ -307,10 +312,9 @@ class Surface(object):
         the surface). See also Surface.reindex_for() method. 
 
         Args: 
-            space: ImageSpace object large enough to contain the surface
-            affine: 4x4 np.array representing transformation into the reference
-                space, in world-world mm terms (not FLIRT scaled-voxels). See
-                utils._FLIRT_to_world for help. Pass None to represent identity.
+            space (ImageSpace): containing the surface
+            struct2ref (np.array): transformation into the reference
+                space, in world-world mm terms (not FLIRT convention)
 
         Updates: 
             self.points: converted into voxel coordinates for the space
@@ -439,7 +443,7 @@ class Surface(object):
         """Apply affine transformation (4x4 array) to surface coordinates"""
 
         self.points = (utils.affineTransformPoints(
-            self.points, transform).astype(np.float32))
+            self.points, transform).astype(NP_FLOAT))
 
 
     def form_associations(self, cores=mp.cpu_count()):
@@ -466,7 +470,7 @@ class Surface(object):
         workerFunc = functools.partial(core._formAssociationsWorker, 
             self.tris, self.points, size)
 
-        if cores > 1:
+        if (cores > 1) and (self._use_mp):
             chunks = utils._distributeObjects(range(self.tris.shape[0]), cores)
             with mp.Pool(cores) as p:
                 worker_assocs = p.map(workerFunc, chunks, chunksize=1)
@@ -497,7 +501,7 @@ class Surface(object):
             (points, tris) tuple of re-indexed points/tris. 
         """
 
-        points = np.empty((0, 3), dtype=np.float32)
+        points = np.empty((0, 3), dtype=NP_FLOAT)
         tris = np.empty((len(tri_inds), 3), dtype=np.int32)
         pointsLUT = []
 
@@ -589,7 +593,7 @@ class Surface(object):
 
         worker = functools.partial(core._voxelise_worker, self)
 
-        if cores > 1: 
+        if (cores > 1) and (self._use_mp): 
 
             # Share out the rays and subset of the overall dimension range
             # amongst pool workers
@@ -629,22 +633,61 @@ class Surface(object):
 
         return igl.adjacency_matrix(self.tris)
 
-    def mesh_laplacian(self):
+    def mesh_laplacian(self, distance_weight=None):
         """
         Mesh Laplacian operator for this surface, as a scipy sparse matrix 
         of size n_points x n_points. Elements on the diagonal are negative 
         and off-diagonal elements are positive. All neighbours are weighted 
         with value 1 (ie, equal weighting ignoring distance). 
+
+        Args: 
+            distance_weight (int): apply inverse distance weighting, default 
+                None (do not weight, all values are unity), whereas positive
+                values will weight elements by 1 / d^n, where d is Euclidean 
+                distance between vertices. 
+
+        Returns: 
+            sparse CSR matrix
         """
 
-        adj = self.adjacency_matrix()
-        central_weight = np.sum(adj, axis=1)
-        laplacian = np.diagflat(central_weight)
+        adj = self.adjacency_matrix().tocsr().astype(NP_FLOAT)
+
+        if distance_weight is not None: 
+            if not (isinstance(distance_weight, int) and (distance_weight > 0)):
+                raise ValueError("distance_weight must be a positive int")
+            
+            # Find all connected pairs of vertices in the upper triangle
+            # of the adjacency matrix. By symmetry of the adjacency matrix, 
+            # we don't need to consider the lower triangle. We use some 
+            # sneaky tricks on the CSR indexing of the adjacency matrix
+            # to work out which columns are non-zero 
+            pairs = []
+            for row in range(adj.shape[0]):
+                for col in adj[row,row:].indices + row:
+                    pairs.append((row,col))
+
+            # Find the distance between connected pairs 
+            pairs = np.array(pairs)
+            dists = np.linalg.norm(self.points[pairs[:,0],:]
+                                    - self.points[pairs[:,1],:], axis=1, ord=2)
+
+            # Apply inverse distance weighting (eg, 1 / d^2), write
+            # them back into the matrix (symmetry again)
+            weight = 1 / (dists ** distance_weight)
+            adj[pairs[:,0], pairs[:,1]] = weight 
+            adj[pairs[:,1], pairs[:,0]] = weight 
+
+        # The diagonal is the negative sum of other elements 
+        dia = adj.sum(1).A.flatten()
+        laplacian = sparse.dia_matrix((dia, 0), shape=(adj.shape))
         laplacian = adj - laplacian
-        assert (laplacian.sum(1) == 0).min(), 'unweighted laplacian'
+
+        assert np.abs(laplacian.sum(1)).max() < 1e-6, 'Unweighted laplacian'
+        assert utils.is_nsd(laplacian), 'Not negative semi-definite'
+        assert utils.is_symmetric(laplacian), 'Not symmetric'
         return laplacian
 
-    def laplace_beltrami(self, area='mayer'):
+    def laplace_beltrami(self, area='mayer', cores=mp.cpu_count()):
         """
         Laplace-Beltrami operator for this surface, as a scipy sparse matrix
         of size n_points x n_points. Elements on the diagonal are negative 
@@ -662,15 +705,31 @@ class Surface(object):
             M = igl.massmatrix(self.points, self.tris, 
                     igl.MASSMATRIX_TYPE_VORONOI)
         elif area == 'mayer':
-            M = core.vtx_tri_weights(self)
+            ncores = cores if self._use_mp else 1 
+            M = core.vtx_tri_weights(self, ncores)
             M = sparse.diags(np.squeeze(M.sum(1).A))
+        else: 
+            raise ValueError("Area must be barycentric/voronoi/mayer.")
 
         L = igl.cotmatrix(self.points, self.tris)
         lbo = (M.power(-1)).dot(L)
         assert (np.abs(lbo.sum(1).A) < 1e-2).all(), 'Unweighted LBO matrix'
         return lbo
 
+    def edges(self):
+        """
+        Edge matrix, sized as follows (tris, 3, 3), where the second dimension
+        contains the edges defined as (v1 - v0), (v2 - v0), (v2 - v1), and 
+        the final dimension contains the edge components in XYZ.  
+        """
+        edge_defns = [ list(e) for e in core.TRI_EDGE_INDEXING ]
+        edges = np.stack([
+            self.points[self.tris[:,e[0]],:] - self.points[self.tris[:,e[1]],:] 
+            for e in edge_defns
+        ], axis=1)
 
+        return edges 
+        
 
 class Patch(Surface):
     """
@@ -725,8 +784,8 @@ class Hemisphere(object):
             raise RuntimeError("Initialise with surface objects")
 
         h = cls.__new__(cls)
-        h.inSurf = insurf
-        h.outSurf = outsurf
+        h.inSurf = copy.deepcopy(insurf)
+        h.outSurf = copy.deepcopy(outsurf)
         h.PVs = None 
         h.side = side 
         return h 
@@ -736,7 +795,6 @@ class Hemisphere(object):
         """Iterator over the inner/outer surfaces"""
         return [self.inSurf, self.outSurf]
 
-
     def surf_dict(self):
         """Return surfs as dict with appropriate keys (eg LPS)"""
         return {self.side + 'WS': self.inSurf, 
