@@ -7,14 +7,15 @@ import subprocess
 import sys
 import shutil
 import warnings 
-import time
 import multiprocessing
 import copy 
 
 import numpy as np 
-import nibabel
 from fsl.wrappers import fsl_anat
-import regtricks as rt 
+import regtricks as rt
+from scipy import sparse
+
+NP_FLOAT = np.float32
 
 STRUCTURES = ['L_Accu', 'L_Amyg', 'L_Caud', 'L_Hipp', 'L_Pall', 'L_Puta', 
                 'L_Thal', 'R_Accu', 'R_Amyg', 'R_Caud', 'R_Hipp', 'R_Pall', 'R_Puta', 
@@ -250,7 +251,7 @@ def affineTransformPoints(points, affine):
     # then re-transpose and drop 4th column  
     transfd = np.ones((points.shape[0], 4))
     transfd[:,0:3] = points
-    transfd = np.matmul(affine, transfd.T).astype(np.float32)
+    transfd = np.matmul(affine, transfd.T).astype(NP_FLOAT)
     return np.squeeze(transfd[0:3,:].T)
 
 
@@ -366,7 +367,7 @@ def enforce_and_load_common_arguments(func):
 
     Required args:
         ref: path to a reference image in which to operate 
-        struct2ref: path to file or 4x4 array representing transformation
+        struct2ref: path/np.array/Registration representing transformation
             between structural space (that of the surfaces) and reference. 
             If given as 'I', identity matrix will be used. 
 
@@ -385,14 +386,11 @@ def enforce_and_load_common_arguments(func):
         a modified copy of kwargs dictionary passed to the caller
     """
     
-    def enforcer(**kwargs):
+    def enforcer(ref, struct2ref, **kwargs):
 
         # Reference image path 
-        if not kwargs.get('ref'):
-            raise RuntimeError("Path to reference image must be given")
-
-        if (type(kwargs['ref']) is str) and not op.isfile(kwargs['ref']):
-            raise RuntimeError("Reference image %s does not exist" % kwargs['ref'])
+        if (not isinstance(ref, rt.ImageSpace)): 
+            ref = rt.ImageSpace(ref)
 
         # If given a anat_dir we can load the structural image in 
         if kwargs.get('anat'):
@@ -422,41 +420,38 @@ def enforce_and_load_common_arguments(func):
                     raise RuntimeError("Could not find T1.nii.gz in the anat dir")
 
  
-        # Structural to reference transformation. Either as array or path
-        # to file containing matrix
-        if not any([type(kwargs.get('struct2ref')) is str, 
-            type(kwargs.get('struct2ref')) is np.ndarray]):
-            raise RuntimeError("struct2ref transform must be given (either path", 
-                "or np.array object)")
+        # Structural to reference transformation. Either as array, path
+        # to file containing matrix, or regtricks Registration object 
+        if not any([type(struct2ref) is str, type(struct2ref) is np.ndarray,
+                    type(struct2ref) is rt.Registration ]):
+            raise RuntimeError("struct2ref transform must be given (either path,", 
+                " np.array or regtricks Registration object)")
 
         else:
-            s2r = kwargs['struct2ref']
+            s2r = struct2ref
 
             if (type(s2r) is str): 
                 if s2r == 'I':
                     matrix = np.identity(4)
                 else:
-                    _, matExt = op.splitext(kwargs['struct2ref'])
+                    _, matExt = op.splitext(s2r)
 
                     try: 
                         if matExt in ['.txt', '.mat']:
-                            matrix = np.loadtxt(kwargs['struct2ref'], 
-                                dtype=np.float32)
+                            matrix = np.loadtxt(s2r, 
+                                dtype=NP_FLOAT)
                         elif matExt in ['.npy', 'npz', '.pkl']:
-                            matrix = np.load(kwargs['struct2ref'])
+                            matrix = np.load(s2r)
                         else: 
-                            matrix = np.fromfile(kwargs['struct2ref'], 
-                                dtype=np.float32)
+                            matrix = np.fromfile(s2r, 
+                                dtype=NP_FLOAT)
 
                     except Exception as e:
                         warnings.warn("""Could not load struct2ref matrix. 
                             File should be any type valid with numpy.load().""")
                         raise e 
 
-                kwargs['struct2ref'] = matrix
-
-        if not kwargs['struct2ref'].shape == (4,4):
-            raise RuntimeError("struct2ref must be a 4x4 matrix")
+                struct2ref = matrix
 
         # If FLIRT transform we need to do some clever preprocessing
         # We then set the flirt flag to false again (otherwise later steps will 
@@ -467,9 +462,13 @@ def enforce_and_load_common_arguments(func):
                 raise RuntimeError("If using a FLIRT transform, the path to the"
                     " structural image must also be given")
             
-            kwargs['struct2ref'] = (rt.Registration.from_flirt(kwargs['struct2ref'], 
-                                        kwargs['struct'], kwargs['ref'])).src2ref
+            struct2ref = rt.Registration.from_flirt(struct2ref, 
+                                        kwargs['struct'], ref).src2ref
             kwargs['flirt'] = False 
+        elif isinstance(struct2ref, rt.Registration): 
+            struct2ref = struct2ref.src2ref 
+        
+        assert isinstance(struct2ref, np.ndarray), 'should have cast struc2ref to np.array'
 
         # Processor cores
         if not kwargs.get('cores'):
@@ -490,15 +489,12 @@ def enforce_and_load_common_arguments(func):
             except:
                 raise RuntimeError("-super must be a value or list of 3" + 
                     " values of int type")
-        
-            kwargs['super'] = sup.astype(np.int8)
-            print("Using manual supersampling factor", kwargs['super'])
 
-        return kwargs
+        return ref, struct2ref, kwargs
 
-    def common_args_enforced(**kwargs):
-        enforced = enforcer(**kwargs)
-        return func(**enforced)
+    def common_args_enforced(ref, struct2ref, **kwargs):
+        ref, struct2ref, enforced = enforcer(ref, struct2ref, **kwargs)
+        return func(ref, struct2ref, **enforced)
 
     return common_args_enforced
 
@@ -541,3 +537,10 @@ def sparse_normalise(mat, axis, threshold=1e-6):
     sums = matrix.sum(axis).A.flatten()
     assert np.all(np.abs((sums[sums > 0] - 1)) < 1e-6), 'Did not normalise to 1'
     return constructor(matrix)
+
+
+def is_symmetric(a, tol=1e-9): 
+    return not (np.abs(a - a.T) > tol).max()
+
+def is_nsd(a):
+    return not (sparse.linalg.eigs(a)[0] > 0).any()
